#define  USE_HY17M28_2M
/******************************************************************************
 * -------------------------------------------------------------------------- *
 * Copyright 2021 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M28                                                           *
 * Program Description:
 * --------------------
 * Thermistor(RS) : AI7-VSS,  min=97k, typ=100k, max=103k
 * RF : Rinp-Rinn, 100k
 * Thermopile1(S1) : AI0-AI1
 * Thermopile2(S2) : AI2-AI3
 * Environment Range(Thermistor) : 21~35C
 * Target Range(Thermopile) : 20~40C
 * RS_measure;      RS resistance. unit : ohm
 * TH_temperature;  TH temperature. unit : temp
 * ADC_RS;          AI7-VSS ADC Count
 * TP_voltage;      TP voltage. unit : uV
 * TP_temperature;  TP temperatur. unit : temp
 * TP2_voltage;      TP2 voltage. unit : uV
 * TP2_temperature;  TP2 temperatur. unit : temp
 * ADC_TP;          AI0-AI1 ADC Count
 * ADC_TP2;          AI2-AI3 ADC Count
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <INT.h>
#include <CLK.h>
#include <RST.h>
#include <UART.h>
#include <PWR.h>
#include <ADC.h>
#include <GPIO.h>
#include <LNPGA.h>

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M28_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M28_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M28_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M28_16M
#define HAO_16MHZ
#endif

#define UART_Port GTX_PT13
#define  Uart_RX_BufferSize 10
#define  Uart_TX_BufferSize 10
//#define UART_ABD

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
volatile typedef union _Flag
{
 unsigned int  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_UART_TxDone:1;
    unsigned b_UART_RxDone:1;
    unsigned Reserved:13;
  };
} Flag;

//signed long
//range 8388607~-8388608 (dec)
//range 0x007F FFFF ~ 0xFF80 0000 (hex)
signed long  ADC_ISRTemp;
signed long  ADCData1;
Flag   Flagbits;
unsigned char UartTxBuffer[Uart_TX_BufferSize]={0};
unsigned char UartRxBuffer[Uart_RX_BufferSize]={0};
unsigned char UartTxIndex,UartTxLength;
unsigned char UartRxIndex,UartRxLength;

// Thermistor //
float RS_25C;
float RF_25C;
float RS_measure;
float RS_kth;

float ADC_RS;     //ADC_RS=External Thermistor 100k
float ADC_RS_P, ADC_RS_N;
float ADC_RF;     //ADC_RF=Internal 100k
float ADC_RF_P, ADC_RF_N;
unsigned int TH_temperature;
unsigned int TP_temperature;
unsigned int TP2_temperature;
int	J_voltage;

// Thermopile //
float Target_voltage;
float TP_gain;
float TP_voltage;

float Target2_voltage;
float TP2_gain;
float TP2_voltage;

float ADC_TP;   //S1(Thermopile1)
float ADC_TP_P, ADC_TP_N;

float ADC_TP2;   //S2(Thermopile2)
float ADC_TP2_P, ADC_TP2_N;


/*----------------------------------------------------------------------------*/
/* Thermistor Table (OTP-638D2)                                               */
/*----------------------------------------------------------------------------*/
//unit : temperature
unsigned int TH[15]={
21,22,23,24,25,
26,27,28,29,30,
31,32,33,34,35
};

//unit : ohm
unsigned long TH_ohm[15]={
119038,113928,109059,104420,100000,  //Thermistor temperature : 21,22,23,24,25,
95788,91775,87950,84305,80830,       //Thermistor temperature : 26,27,28,29,30,
77517,74357,71342,68466,65720        //Thermistor temperature : 31,32,33,34,35
};

/*----------------------------------------------------------------------------*/
/* Thermopile Table (OTP-638D2)                                               */
/*----------------------------------------------------------------------------*/
//unit : temperature
unsigned int TP[21]={
20,21,22,23,24,
25,26,27,28,29,
30,31,32,33,34,
35,36,37,38,39,
40
};


//unit : uV
//OTP-638D2, absolute 25C table
unsigned int TP_V[21]={
1628,1722,1815,1909,2003,      //Thermopile voltage : 20~24C
2097,2191,2285,2379,2473,      //Thermopile voltage : 25~29C
2567,2665,2765,2865,2966,      //Thermopile voltage : 30~34C
3067,3170,3273,3378,3483,      //Thermopile voltage : 35~39C
3590                                          //Thermopile voltage : 40C
};


/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int del);
void CLOCKInit(void);
void UARTInit(void);
void SendUART(signed long  data);
void SendUART_HEADER(unsigned char data);

void Inital_ADC_TH(void);
void Inital_ADC_TP(void);
void Measure_ADC_TH(void);
void Measure_ADC_TP(void);
void Measure_ADC_TP2(void);

void Temptable_TH(void);
void Temptable_TP(void);
void Temptable_TP2(void);
void Cold_Junction(void);

/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
  UartTxIndex=0;
  UartRxIndex=0;
  Flagbits.b_ADCdone=0;
  Flagbits.b_UART_TxDone=0;
  Flagbits.b_UART_RxDone=0;

  RS_25C=100000;         //Assume RS=100k
  RF_25C=100000;         //Assume RF=100k

  CLOCKInit();
  UARTInit();
  Flagbits.b_UART_TxDone=1; //Avoid UART Enable error on first data
  GIE_Enable();

  while(1)
  {

    //Thermistor
    Inital_ADC_TH();
    Measure_ADC_TH();			       //ADC raw count to R, get RS_measure and RS_kth ( unit :R )
    Temptable_TH();			              //R to T, get TH_temperature
    Cold_Junction();			            //T to V, get J_voltage

    //Thermopile
    Inital_ADC_TP();

    Measure_ADC_TP();			                      //ADC raw count to V, get TP_voltage=Thermopile measure voltage( unit :uV )
    Target_voltage=TP_voltage+J_voltage;   //Target_voltage=TP_voltage+J_voltage
    Temptable_TP();			                              //V to T, get TP_temperature Tempurature

    Measure_ADC_TP2();			                      //ADC raw count to V, get TP_voltage=Thermopile measure voltage( unit :uV )
    Target2_voltage=TP2_voltage+J_voltage;   //Target_voltage=TP_voltage+J_voltage
    Temptable_TP2();			                              //V to T, get TP2_temperature Tempurature

    ADC_INT_Disable();
    //UART send data
    SendUART_HEADER(0x1);  //Ch1
    SendUART(TP_temperature);
    SendUART_HEADER(0x2);  //Ch2
    SendUART(TP2_temperature);
    ADC_INT_Enable();
  }


}

/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{
  NOP();
  //ADC Event
  if(ADCIF_IsFlag())
  {
    ADCIF_ClearFlag();
    ADC_ISRTemp=ADCR; //ADC_GetData();
    Flagbits.b_ADCdone=1;
  }

  //UART RX Event
  if(UART0_INT_TXIsFlag())
  {
    if(Flagbits.b_UART_TxDone==0)
    {
      TX0R=UartTxBuffer[UartTxIndex++];
      UART0_INT_TXClearFlag();
      if(UartTxIndex>=UartTxLength)
      {
        UART0_INT_TXEnable();    //TXIE=1b
        UART0_INT_RCEnable();  //RCIE=1b
        Flagbits.b_UART_TxDone=1;
        UartTxIndex=0;
      }
    }
    if(Flagbits.b_UART_TxDone==1)
    {
      UART0_INT_TXDisable();    //TXIE=0b
      UART0_INT_RCEnable();  //RCIE=1b
      UART0_INT_TXClearFlag();
    }
  }


}
/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int del)
{
  while(--del)
  NOP();
}
/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif
}

/*----------------------------------------------------------------------------*/
/* UART Init Function                                                          */
/*----------------------------------------------------------------------------*/
void UARTInit(void)
{
  //GPIO Init
  GPIO_GTXSet(UART_Port);  //TX PT1.3 Rx PT1.4

#ifndef UART_ABD
    //BRGR[15:0]=((CPUCK/Baudrate)/4)-1
  #if defined(HAO_2MHZ)
    //((1843000/9600)/4)-1 =47, HAO=1.843M, baudrate=9600
    UART0_Open(47 ,8 ,PARITY_None);
  #endif
  #if defined(HAO_4MHZ)
    //((4147000/9600)/4)-1 =107, HAO=4.147M, baudrate=9600
    UART0_Open(107 ,8 ,PARITY_None);
  #endif
  #if defined(HAO_8MHZ)
    //((8755000/9600)/4)-1 =227, HAO=8.755M, baudrate=9600
    UART0_Open(227 ,8 ,PARITY_None);
  #endif
  #if defined(HAO_16MHZ)
    //((15667000/9600)/4)-1 =407, HAO=15.667M, baudrate=9600
    UART0_Open(407 ,8 ,PARITY_None);
  #endif
#else   
  UART0_Open(0 ,8 ,PARITY_None);
  UART0_WUEDisable();                   // Wake-up disable
  UART0_ABDEnable();                    //Enable Auto Baudrate
  while((BA0CN & 0x01) == 1 );      //Wait for master send 055H
  UART0_INT_RCClearFlag();          //Clear RC interrupt flag
 
#endif
  TXIE_Enable();
  RCIE_Enable();
}

/*----------------------------------------------------------------------------*/
/* Function Name: void SendUART(int data)                                     */
/* Description  : UART send data.                                             */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void SendUART(signed long  data)
{
  //display range +8388607~-8388608 (dec)
  ADCData1=data;
  if((ADCData1<0)||(ADCData1>0x80000000))  // plus-minus sign judgment
  {
	  ADCData1=~ADCData1;
	  ADCData1++;
      UartTxBuffer[0]='-';
  }
  else
  {
    UartTxBuffer[0]='+';
  }
  UartTxBuffer[7]=ADCData1%10 | '0';  //unit
  ADCData1=ADCData1/10;
  UartTxBuffer[6]=ADCData1%10 | '0';  //unit
  ADCData1=ADCData1/10;
  UartTxBuffer[5]=ADCData1%10 | '0';  //ten
  ADCData1=ADCData1/10;
  UartTxBuffer[4]=ADCData1%10 | '0';  //hundred
  ADCData1=ADCData1/10;
  UartTxBuffer[3]=ADCData1%10 | '0';  //thousand
  ADCData1=ADCData1/10;
  UartTxBuffer[2]=ADCData1%10 | '0';  //ten thousand
  ADCData1=ADCData1/10;
  UartTxBuffer[1]=ADCData1%10 | '0';  //hundred thousand
  ADCData1=ADCData1/10;
  UartTxBuffer[8]='\r';
  UartTxBuffer[9]='\n';
  Flagbits.b_UART_TxDone=0;
  UartTxLength=10;
  UartTxIndex=0;
  TXIE_Enable();
  RCIE_Enable();
  while(!Flagbits.b_UART_TxDone);  //If  Flagbits.b_UART_TxDone=DISABLE, stop at here
}

/*----------------------------------------------------------------------------*/
/* Inital_ADC_TH Init Function                                                */
/*----------------------------------------------------------------------------*/
void Inital_ADC_TH(void)
{
  //Inital_ADC_TH, to measure internal 100k
  //==========Setup Power=====================================
  PWR_BGREnable();			//bandgap Vref Enable
  PWR_VDDAOpen(LDOC_2V4);	//VDDA Enable, refer to the Datasheet for LDOC typical voltage
  PWR_VCMSSelect(VCMS_ACMint);
  PWR_ACMintOpen(SELVIN_V12);

  //==========Setup ADC =================================
#if defined(HAO_2MHZ)
  //HAO=2MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV2,INP_Rinp,INN_Rinn,VRH_V12SEL,VRL_VSS,ADGN_1,VREGN_DIV1,DCSET_P0,OSR_16384);
#endif

#if defined(HAO_4MHZ)
  //HAO=4MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV4,INP_Rinp,INN_Rinn,VRH_V12SEL,VRL_VSS,ADGN_1,VREGN_DIV1,DCSET_P0,OSR_16384);
#endif

#if defined(HAO_8MHZ)
  //HAO=8MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV8,INP_Rinp,INN_Rinn,VRH_V12SEL,VRL_VSS,ADGN_1,VREGN_DIV1,DCSET_P0,OSR_16384);
#endif

#if defined(HAO_16MHZ)
  //HAO=15.6MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV16,INP_Rinp,INN_Rinn,VRH_V12SEL,VRL_VSS,ADGN_1,VREGN_DIV1,DCSET_P0,OSR_16384);
#endif

  ADC_VRHV12SEL(V12SEL_REFOI); //VRH=V12SEL=REFOI
  ADC_INPBufferEnable();
  ADC_INNBufferEnable();
  PWR_ACMintOpen(SELVIN_VDDADIV2);
  PWR_REFOOpen(REFOS_1V2,REFO1_NOPAD); //REFOS=1.2V,ENREFO=1b(Not output to pad).
  PGA_NTCRefResistance(VRPOS_Rinp);  //ADC Vin(Rinp)=internal 100k
  PGA_Disable();

}

/*----------------------------------------------------------------------------*/
/* Function Name: Measure_ADC_TH()                                            */
/* Description  : ADC thermistor channel measurement.                         */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Measure_ADC_TH(void)
{

  ////get ADC_RF_P
  ADC_VINSelect(INP_Rinp,INN_Rinn);  //ADC Vin=internal 100k=Rinp-Rinn
  ADC_CMFREnable();
  Flagbits.b_ADCdone=0;
  while(Flagbits.b_ADCdone==0);
  ADC_RF_P=ADC_ISRTemp;

  ////get ADC_RF_N
  ADC_VINSelect(INP_Rinn,INN_Rinp);  //ADC Vin=internal 100k=Rinn-Rinp
  ADC_CMFREnable();
  Flagbits.b_ADCdone=0;
  while(Flagbits.b_ADCdone==0);
  ADC_RF_N=ADC_ISRTemp;

  ADC_RF=(ADC_RF_P-ADC_RF_N)/2;  //get ADC Raw count on internal 100k=RF

  ////get ADC_RS_P
  ADC_VINSelect(INP_AI7,INN_VSS);  //ADC Vin=Thermistor 100k=AI7-VSS
  ADC_INXSelect(INX_NORMAL);
  ADC_INPBufferEnable();  //ADSIPB=1b (when INX=00b, INP=AI7, through input buffer)
  ADC_INNBufferDisable(); //ADSINB=0b (when INX=00b, INN=VSS, no through input buffer)

  ADC_CMFREnable();
  Flagbits.b_ADCdone=0;
  while(Flagbits.b_ADCdone==0);
  ADC_RS_P=ADC_ISRTemp;

  ////get ADC_RS_N
  ADC_INXSelect(INX_CROSS);
  ADC_INPBufferDisable();  //ADSIPB=0b (when INX=11b, INP=VSS, no through input buffer)
  ADC_INNBufferEnable();   //ADSINB=1b (when INX=11b, INN=AI7, through input buffer)

  ADC_CMFREnable();
  Flagbits.b_ADCdone=0;
  while(Flagbits.b_ADCdone==0);
  ADC_RS_N=ADC_ISRTemp;

  ADC_RS=(ADC_RS_P-ADC_RS_N)/2;  //get ADC Raw count on Thermistor(RS)

  RS_measure=(ADC_RS*RF_25C)/ADC_RF;
  RS_kth = RS_measure*RF_25C/RS_25C;

}

/*----------------------------------------------------------------------------*/
/* Inital_ADC_TP Init Function                                                */
/*----------------------------------------------------------------------------*/
void Inital_ADC_TP(void)
{
  //==========Setup Power=====================================
  PWR_BGREnable();			//bandgap Vref Enable
  PWR_VDDAOpen(LDOC_2V4);	//VDDA Enable, refer to the Datasheet for LDOC typical voltage
  PWR_VCMSSelect(VCMS_ACMint);
  PWR_ACMintOpen(SELVIN_V12);

  //==========Setup ADC =================================
#if defined(HAO_2MHZ)
  //HAO=2MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV2,INP_OPOI,INN_V12,VRH_V12SEL,VRL_VSS,ADGN_1,VREGN_DIV1,DCSET_P0,OSR_16384);
#endif

#if defined(HAO_4MHZ)
  //HAO=4MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV4,INP_OPOI,INN_V12,VRH_V12SEL,VRL_VSS,ADGN_1,VREGN_DIV1,DCSET_P0,OSR_16384);
#endif

#if defined(HAO_8MHZ)
  //HAO=8MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV8,INP_OPOI,INN_V12,VRH_V12SEL,VRL_VSS,ADGN_1,VREGN_DIV1,DCSET_P0,OSR_16384);
#endif

#if defined(HAO_16MHZ)
  //HAO=15.6MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV16,INP_OPOI,INN_V12,VRH_V12SEL,VRL_VSS,ADGN_1,VREGN_DIV1,DCSET_P0,OSR_16384);
#endif
  ADC_VRHV12SEL(V12SEL_V12); //VRH=V12SEL=REFOI
  PWR_REFOOpen(REFOS_1V2,REFO1_NOPAD); //REFOS=1.2V,ENREFO=1b(Not output to pad).
  PGA_NTCRefResistance(VRPOS_Rinp);
  PGA_TR1ChopOFF();
  PGA_TR2ChopOFF();
  PGA_Enable();
  PGA_InputSelect(INPS_TR1);  //select S1 channel, measure Thermopile1
  PWR_REFOISource(VRSEL_ACMint);

}

/*----------------------------------------------------------------------------*/
/* Function Name: Measure_ADC_TP()                                            */
/* Description  : ADC Thermopile measurement.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Measure_ADC_TP(void)
{

  ////get ADC_TP_P
  PGA_InputSelect(INPS_TR1);  //select S1 channel, measure Thermopile1
  PGA_TR1ChopOFF();

  ADC_CMFREnable();
  Flagbits.b_ADCdone=0;
  while(Flagbits.b_ADCdone==0);
  ADC_TP_P=ADC_ISRTemp;

  ////get ADC_TP_N
  PGA_TR1ChopON();

  ADC_CMFREnable();
  Flagbits.b_ADCdone=0;
  while(Flagbits.b_ADCdone==0);
  ADC_TP_N=ADC_ISRTemp;

  ADC_TP=(ADC_TP_P-ADC_TP_N)/2;

  //b1.176mVq67880 count, ]0mVq500countADC count
  //TP_gain=68380=(67880+500)/1176;  //68380 means ADC count, 1176 means 1.176mV
  //unit : uV
  TP_gain=(68380/1176)*1000;    //68380 means ADC count, 1176 means 1.176m
  //unit : uV
  TP_voltage=(ADC_TP/TP_gain)*1000;

}

/*----------------------------------------------------------------------------*/
/* Function Name: Measure_ADC_TP2()                                            */
/* Description  : ADC Thermopile measurement.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Measure_ADC_TP2(void)
{
  ////get ADC_TP2_P
  PGA_InputSelect(INPS_TR2);      //select S2 channel, measure Thermopile2
  PGA_TR2ChopOFF();   //TR2Chop=0b

  ADC_CMFREnable();
  Flagbits.b_ADCdone=0;
  while(Flagbits.b_ADCdone==0);
  ADC_TP2_P=ADC_ISRTemp;


  ////get ADC_TP2_N
  PGA_TR2ChopON();

  ADC_CMFREnable();
  Flagbits.b_ADCdone=0;
  while(Flagbits.b_ADCdone==0);
  ADC_TP2_N=ADC_ISRTemp;

  ADC_TP2=(ADC_TP2_P-ADC_TP2_N)/2;

  //b1.176mVq90000 count, ]0mVq500countADC count
  //TP2_gain=90500=(90000+500)/1176;  //90500 means ADC count, 1176 means 1.176mV
  //unit : uV
  TP2_gain=(90500/1176)*1000;    //90500 means ADC count, 1176 means 1.176m
  //unit : uV
  TP2_voltage=(ADC_TP2/TP2_gain)*1000;

}
/*----------------------------------------------------------------------------*/
/* Function Name: Temptable_TH()                                              */
/* Description  : Thermistor temperature table searching                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Temptable_TH(void)
{
  float count1;
  float tempH,tempL;
  char count2;
  unsigned char a=0;
  unsigned char b=0;
  unsigned char max=14,min=0;  //TH[15]
  a=(max+min)/2;
  b=a+1;
  while(1)
  {
    tempH=TH_ohm[a+1];
    tempL=TH_ohm[a];
    if(RS_kth>tempH&&RS_kth<=tempL)
    break;
    else
    {
      b=b/2;
      if(b==0)
      b=1;
      if(RS_kth>tempL)
      a=a-b;
      else if(RS_kth<=tempH)
      a=a+b;
      if(a<min||a>max)
      break;
    }
  }
  count1=(tempL-tempH)/100;              //Note : /100 rR?B?K100??, count1rV?Y?G
  count2=(RS_kth-tempL)/count1;          //count2 rY??nO?
  TH_temperature=TH[a]*100-count2;  //Note : *100 rR?B?K100??, ԕx?Rx?r?Ax
  if(a==255)                             //a=a-b, but a=0 and b=1, a will be 255
  (TH_temperature=TH[min]*100);          //to show TH[min]
  if(a>max && a!=255)
  (TH_temperature=TH[max]*100-count2);   //to show TH[max]
}
/*----------------------------------------------------------------------------*/
/* Function Name: Temptable_TP()                                              */
/* Description  : Thermopile temperature table searching                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Temptable_TP(void)
{
  float count1;
  float tempH,tempL;
  char count2;
  unsigned char a=0;
  unsigned char b=0;
  unsigned char max=20,min=0;  //TP[21]=20C~40C
  a=(max+min)/2;
  b=a+1;
  while(1)
  {
    tempH=TP_V[a+1];
    tempL=TP_V[a];
    if(Target_voltage<tempH&&Target_voltage>=tempL)
    break;
    else
    {
      b=b/2;
      if(b==0)
      b=1;
      if(Target_voltage<tempL)
      a=a-b;
      else if(Target_voltage>=tempH)
      a=a+b;
      if(a<min||a>max)
      break;
    }
  }
  count1=(tempH-tempL)/100;
  count2=(Target_voltage-tempL)/count1;
  TP_temperature=TP[a]*100+count2;
  if(a==255)                          //a=a-b, but a=0 and b=1, a will be 255
  (TP_temperature=TP[min]*100);       //to show TP[min]
  if(a>max && a!=255)
  (TP_temperature=TP[max]*100);       //to show TP[max]
}

/*----------------------------------------------------------------------------*/
/* Function Name: Temptable_TP2()                                              */
/* Description  : Thermopile temperature table searching                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Temptable_TP2(void)
{
  float count1;
  float tempH,tempL;
  char count2;
  unsigned char a=0;
  unsigned char b=0;
  unsigned char max=20,min=0;  //TP[21]=20C~40C
  a=(max+min)/2;
  b=a+1;
  while(1)
  {
    tempH=TP_V[a+1];
    tempL=TP_V[a];
    if(Target2_voltage<tempH&&Target2_voltage>=tempL)
    break;
    else
    {
      b=b/2;
      if(b==0)
      b=1;
      if(Target2_voltage<tempL)
      a=a-b;
      else if(Target2_voltage>=tempH)
      a=a+b;
      if(a<min||a>max)
      break;
    }
  }
  count1=(tempH-tempL)/100;
  count2=(Target2_voltage-tempL)/count1;
  TP2_temperature=TP[a]*100+count2;
  if(a==255)                          //a=a-b, but a=0 and b=1, a will be 255
  (TP2_temperature=TP[min]*100);       //to show TP[min]
  if(a>max && a!=255)
  (TP2_temperature=TP[max]*100);       //to show TP[max]
}
/*----------------------------------------------------------------------------*/
/* Function Name: Cold_Junction()                                             */
/* Description  :                                                             */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Cold_Junction(void)
{
  int tempH,tempL;
  float Temp_remain;
  unsigned char a=0;
  unsigned char b=0;
  unsigned char max=20,min=0;  //TP[21]=20C~40C
  a=(max+min)/2;
  b=a+1;
  while(1)
  {
    tempH=TP[a+1];
    tempL=TP[a];
    if((TH_temperature/100)<tempH&&(TH_temperature/100)>=tempL)
    break;
    else
    {
      b=b/2;
      if(b==0)
      b=1;
      if((TH_temperature/100)<tempL)
      a=a-b;
      else if((TH_temperature/100)>=tempH)
      a=a+b;
      if(a<min||a>max)
      break;
    }
  }
  Temp_remain=(TH_temperature%100)*(TP_V[a+1]-TP_V[a])/100;
  J_voltage=TP_V[a]+Temp_remain;
  if(a==255)                                  //a=a-b, but a=0 and b=1, a will be 255
  (J_voltage=TP_V[min]);                      //to show TP_V[min]
  if(a>max && a!=255)
  (J_voltage=TP_V[max]+Temp_remain);          //to show TP_V[max]
}

/*----------------------------------------------------------------------------*/
/* Function Name: void SendUART_HEADER(unsigned char  data)                                     */
/* Description  : UART send data.                                             */
/* Arguments    : None.                                                       */
/* Return Value : None.         TA1CIE_Enable();                                              */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void SendUART_HEADER(unsigned char  data)
{

  if(data==0x1)
  {
    UartTxBuffer[0]=0xA1;
    Flagbits.b_UART_TxDone=0;
    UartTxLength=1;
    UartTxIndex=0;
    TXIE_Enable();
    while(!Flagbits.b_UART_TxDone);  //If  Flagbits.b_UART_TxDone=DISABLE, stop at here
  }

  else if(data==0x2)
  {
    UartTxBuffer[0]=0xA2;
    Flagbits.b_UART_TxDone=0;
    UartTxLength=1;
    UartTxIndex=0;
    TXIE_Enable();
    while(!Flagbits.b_UART_TxDone);  //If  Flagbits.b_UART_TxDone=DISABLE, stop at here
  }


}
/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
